#include <grape.h>


/* grape-method "new-instance"
 *              "delete"
 *
 * Call              : GRAPE(time_object, "new-instance")(name)
 *                     GRAPE(time_object, "delete")();
 *
 * Parameters        : TIME_OBJECT *time_object
 *                     char *name
 *
 * Description       : "new-instance" creates an instance of class
 *                     Time_Object with name "name". Only the descriptor
 *                     "list_of_task" is allocated completely, status
 *                     has to be allocated separatedly using "list-alloc".
 *
 *                     "delete" deletes an instance of class Time_Object.
 *                     All data is deleted using "list-free".
 *
 * Author            : Konrad Polthier, Bernd Oberknapp
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_OBJECT *time_object_new_instance(char *name)
{
  TIME_OBJECT *self;
  CLASS *class;

  class = (CLASS *)START_METHOD(G_CLASS);
  ASSURE(class, "", END_METHOD(NULL));

  self = (TIME_OBJECT *)GRAPE(class, "^new-instance")(name);
  ASSURE(self, "Can't create time_object instance", END_METHOD(NULL));

  self->task = 0;
  self->task_mask = tskGeometry;
  self->frame         = 0;
  self->num_of_frames = 0;
  self->status        = NULL;
  self->time = (double *)GRAPE(Time_Object, "get-time")();
  self->time_parm  = (TIME_PARM *)
    GRAPE(Time_Parm, "new-instance")("time_parm");
  self->geometry      = NULL;
  self->inter         = NULL;
  self->tree          = NULL;

  END_METHOD(self);
}


TIME_OBJECT *time_object_free()
{
  TIME_OBJECT *self;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  if(self->class == Time_Object)
    GRAPE(self, "list-free")();

  if(self->time_parm)
    self->time_parm = (TIME_PARM *)GRAPE(self->time_parm, "delete")();

  self->time = NULL;

  GRAPE(self, "^free")();

  END_METHOD(NULL);
}


/* grape-method "list-alloc"
 *              "list-free"
 *
 * Call              : GRAPE(time_object, "list-alloc")()
 *                     GRAPE(time_object, "list-free")()
 *
 * Parameters        : TIME_OBJECT *time_object
 *
 * Description       : "list-alloc" allocates and initializes status
 *                     for an instance of class Time_Object
 *                     (num_of_frames has to be initialized!).
 *
 *                     "list-free" deletes all data which has not been
 *                     allocated by "new-instance" including user allo-
 *                     cated geometry, interpol and tree instances.
 *
 * Author            : Bernd Oberknapp, Konrad Polthier
 *
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_OBJECT *time_object_list_alloc()
{
  TIME_OBJECT *self;
  int i;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  self->status = int_alloc(self->num_of_frames);
  for(i = 0; i < self->num_of_frames; i++)
    self->status[i] = NOT_ACTIVE;

  self->inter = (CHAIN *)GRAPE(Chain, "new-instance")("inter");

  END_METHOD(self);
}


TIME_OBJECT *time_object_list_free()
{
  TIME_OBJECT *self;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  if(self->tree)
    self->tree = (SCENE *)GRAPE(self->tree, "delete")();
  if(self->inter)
    self->inter = (CHAIN *)GRAPE(self->inter, "delete")();
  if(self->geometry)
    self->geometry = (CHAIN *)GRAPE(self->geometry, "delete")();
  if(self->time_parm)
    GRAPE(self->time_parm, "list-free")();
  if(self->status)
    self->status = int_free(self->status, self->num_of_frames);

  self->frame = self->num_of_frames = 0;

  END_METHOD(self);
}


TIME_OBJECT *time_object_hardcopy()
{
  TIME_OBJECT *self, *copy;
  int i;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  copy = (TIME_OBJECT *)GRAPE(self, "^hardcopy")();
  ASSURE(copy, "cannot create new time_object", END_METHOD(NULL));

  copy->task = self->task;
  copy->task_mask = self->task_mask; /* should be set anyway */

  copy->frame  = self->frame;
  copy->num_of_frames = self->num_of_frames;

  copy->status = int_alloc(self->num_of_frames);
  for(i = 0; i < self->num_of_frames; i++)
    copy->status[i] = self->status[i];

  copy->time = self->time;

  /* the following should be made "hardcopy" */
  if(self->time_parm)
    ASSIGN(copy->time_parm, self->time_parm);

  if(self->geometry)
    copy->geometry = (CHAIN *)GRAPE(self->geometry, "hardcopy")();

  copy->inter = NULL;
  copy->tree = NULL;

  END_METHOD(copy);
}


TIME_OBJECT *time_object_softcopy(TIME_OBJECT *copy)
{
  TIME_OBJECT *self;
  int i;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  copy = (TIME_OBJECT *)GRAPE(self, "^softcopy")(copy);
  ASSURE(copy, "cannot create new time_object", END_METHOD(NULL));

  copy->task = self->task;
  copy->task_mask = self->task_mask; /* should be set anyway */

  if(copy->num_of_frames &&
     copy->num_of_frames != self->num_of_frames) {
    int_free(copy->status, copy->num_of_frames);
    copy->status = int_alloc(self->num_of_frames);
  }

  copy->frame  = self->frame;
  copy->num_of_frames = self->num_of_frames;

  for(i = 0; i < self->num_of_frames; i++)
    copy->status[i] = self->status[i];

  copy->time = self->time;

  /* the following should be made "softcopy" */
  if(self->time_parm)
    ASSIGN(copy->time_parm, self->time_parm);

  if(self->geometry)
    copy->geometry = (CHAIN *)
      GRAPE(self->geometry, "softcopy")(copy->geometry);

  copy->inter = NULL;
  copy->tree = NULL;

  END_METHOD(copy);
}


/* grape-method "merge-send"
 *
 * Call              : GRAPE (time_object, "merge-send") ();
 *
 * Parameters        : TIME_OBJECT *time_object
 *
 * Description       : Send merge-send to all major geometries under time_object
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1992  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_OBJECT *time_object_merge_send()
{
  TIME_OBJECT *self;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD (NULL));

  if(self->geometry) {
    printf("object = %s\n", self->geometry->object->name);
    GRAPE(self->geometry, "merge-send")();
  }

  if(self->time_parm)
    GRAPE(self->time_parm, "set")(INACTIVE);

  END_METHOD (self);
}


/* grape-method "switch-comp-send"
 *
 * Call              : GRAPE (time_object, "switch-comp-send") ();
 *
 * Parameters        : TIME_OBJECT *time_object
 *
 * Description       : Send switch-send to time_parm
 *
 * Author            : Konrad Polthier
 *
 * Copyright (c) 1993  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_OBJECT *time_object_switch_comp_send()
{
  TIME_OBJECT *self;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD (NULL));

  if(self->time_parm)
    GRAPE(self->time_parm, "switch-send")();

  END_METHOD (self);
}


/* grape-method "close-send"
 *
 * Call              : GRAPE(time_object, "close-send")();
 *
 * Parameters        : TIME_OBJECT *time_object
 *
 * Description       : Send close-send to all major geometries, merge first
 *                     if necessary.
 *
 * Author            : Bernd Oberknapp, Konrad Polthier
 *
 * Copyright (c) 1992-96 by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 */

TIME_OBJECT *time_object_close_send()
{
  TIME_OBJECT *self;

  self = (TIME_OBJECT *)START_METHOD(G_INSTANCE);
  ASSURE (self, "", END_METHOD(NULL));

  if(self->geometry) {
    if(self->geometry->next_scene) {
      printf("time_object_close_send: applying merge first\n");
      GRAPE(self, "merge-send")();
    }
    if(self->geometry->object) {
      printf("object = %s\n", self->geometry->object->name);
      GRAPE(self->geometry->object, "close-send")();
    }
  }

  END_METHOD(self);
}
