/*
 * grape-module "rot2d"
 *
 * Description       : Interactive and  rotate method for class Rot2d.
 *
 *                     "get-info" just adds information about the angle
 *                     and discretisation to the instance info list.
 *
 *                     "follow" defines axis and curve to be shown right
 *                     to the Rot2d instance.
 *
 *                     "rotate" computes the surface of revolution from
 *                     the given data.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1995  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


ROT2D *rot2d_get_info(XLIST *list)
{
  ROT2D *self;
  char str[MAX_STRLEN];

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * Most of the work is done by the superclass method. It puts
   * information about the name, class, refcount, number of points
   * and so on into the list.
   */
  GRAPE(self, "^get-info")(list);

  /*
   * Just add the angle and the discretization to the list. We
   * also could display information about axis and curve but
   * the user can get this information by changing the object
   * to axis or curve and then clicking on it.
   */
  sprintf(str, "angle: %f", self->angle);
  GRAPE(list, "add-to-list")(str);
  sprintf(str, "discr: %d", self->discr);
  GRAPE(list, "add-to-list")(str);

  END_METHOD(self);
}


ROT2D *rot2d_follow(INSTANCE *inst[3][3], char pop_push[3][3])
{
  ROT2D *self;

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * The inst and pop_push arrays define which objects will be
   * shown in the manager if a Rot2d instance is the current
   * object (there are three by three instances possible).
   * In this case the superclass method just initializes the
   * array with zeros, it does not provide its own environment.
   */
  GRAPE(self, "^follow")(inst, pop_push);

  /*
   * If we have axis and/or curve we want is to be shown top right
   * and right of the Rot2d instance. The `d' stands for `down',
   * this means that the Rot2d instance is moved to the left if
   * we click on the axis or curve.
   */
  if(self->axis) {
    inst[0][2] = (INSTANCE *)self->axis;
    pop_push[0][2] = 'd';
  }
  if(self->curve) {
    inst[1][2] = (INSTANCE *)self->curve;
    pop_push[1][2] = 'd';
  }

  END_METHOD(self);
}


ROT2D *rot2d_rotate_send()
{
  ROT2D *self;
  TRIANG1D *curve, *axis;
  MATRIX44 rotseg, rot, prod;
  VEC3 paxis[2], prot;
  int number_of_points;
  int number_of_elements;
  int ulines, vlines;
  int i, j, ind;
  double angle;

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * Before we start computing the surface some check have to be done.
   * Curve and axis must exist and have enough resp. the right number
   * of points. Creating a negative or zero number of segments doesn't
   * make much sense and we need a non-zero angle.
   */
  ASSURE((curve = self->curve) && curve->number_of_points > 1,
	 "rot2d_rotate_send: no curve with enough points",
	 END_METHOD(NULL));
  ASSURE((axis = self->axis) && axis->number_of_points == 2,
	 "rot2d_rotate_send: no correct axis found",
	 END_METHOD(NULL));
  ASSURE(self->discr > 0,
	 "rot2d_rotate_send: need discr > 0",
	 END_METHOD(NULL));
  ASSURE(self->angle != 0.0,
	 "rot2d_rotate_send: need angle != 0",
	 END_METHOD(NULL));

  /*
   * First the number of points and elements is computed. For discr
   * segments we have discr + 1 lines, the number of lines in the
   * other direction is given by the curve.
   */
  ulines = self->discr + 1;
  vlines = curve->number_of_points;

  /*
   * A grid with each quadrilateral divided into two triangles
   * is used, so we need these numbers of points an elements:
   */
  number_of_points = ulines * vlines;
  number_of_elements = 2 * (ulines - 1) * (vlines - 1);

  /*
   * Now check if this Rot2d instance has enough memory to store
   * the new triangulation. We compare both the computed number
   * of points and triangles to the maximum number available.
   */
  if(self->max_number_of_points < number_of_points ||
     self->max_number_of_elements < number_of_elements)
    /*
     * If there isn't enough memory we just extend the old memory
     * lists by calling "list-alloc". We will overwrite the old data
     * which is preserved by "list-alloc" later.
     */
    GRAPE(self, "list-alloc")(number_of_points, number_of_elements);

  /*
   * The information about how much of the memory of the instance
   * is used has to be stored in the instance, "list-alloc" only
   * sets the maximum number of points and elements to the array
   * sizes.
   */
  self->number_of_points = number_of_points;
  self->number_of_elements = number_of_elements;

  /*
   * Each segment of the surface is angle / discr wide. g_line_rotate
   * computes the rotation matrix for a rotation around and arbitrary
   * axis. Unfortunately we have to copy the end points of the axis
   * because this function needs an array of two VEC3. The other para-
   * meters are the angle and the matrix that has to be computed.
   * The rotseg matrix describes the rotation for one segment, the
   * rotation matrix rot is set to the identity matrix because we
   * start with no rotation.
   */
  angle = (2.0 * M_PI / 360.0) * self->angle / (double)self->discr;
  paxis[0][0] = axis->x[0]; paxis[0][1] = axis->y[0]; paxis[0][2] = axis->z[0];
  paxis[1][0] = axis->x[1]; paxis[1][1] = axis->y[1]; paxis[1][2] = axis->z[1];
  g_line_rotate(paxis, angle, rotseg);
  g_matrix44_set_identity(rot);

  /*
   * Now we compute the points. For each segment was have to transform
   * the points of the curve with the current rotation matrix. Again
   * we have to copy the points from and to the triangulation to use
   * the matrix multiplication function (alternatively we could have
   * done the multiplication by hand). After each segment the rotation
   * matrix is multiplied with the segment rotation matrix to get the
   * matrix needed for the next segment.
   */
  for(ind = 0, i = 0; i < ulines; i++) {
    for(j = 0; j < vlines; j++, ind++) {
      prot[0] = curve->x[j]; prot[1] = curve->y[j]; prot[2] = curve->z[j];
      g_matrix44_mult_vec3(rot, prot);
      self->x[ind] = prot[0]; self->y[ind] = prot[1]; self->z[ind] = prot[2];
    }
    g_matrix44_mult(prod, rotseg, rot);
    g_matrix44_assign(prod, rot);
  }

  /*
   * We also have to define the connectivity of the triangulation
   * (which points belong to which triangle). Again we run through
   * the segments storing two triangles for each quadrilateral.
   * It's really quite simple in this case...
   */
  for(i = 0; i < ulines - 1; i++)
    for(j = 0; j < vlines - 1; j++) {
      ind = (i * (vlines - 1) + j) * 2;
      self->vertex[ind][0] = i * vlines + j;
      self->vertex[ind][1] = (i + 1) * vlines + j;
      self->vertex[ind][2] = i * vlines + j + 1;
      ind++;
      self->vertex[ind][0] = i * vlines + j + 1;
      self->vertex[ind][1] = (i + 1) * vlines + j;
      self->vertex[ind][2] = (i + 1) * vlines + j + 1;
    }

  /*
   * Some methods also need information about the neighbours of the
   * triangles. Computing this directly is no problem, but since
   * we have a method for this we just call it.
   */
  GRAPE(self, "make-neighbour-send")();

  END_METHOD(self);
}
