/*
 * grape-module "rot2d_mem"
 *
 * Description       : Memory management for class Rot2d.
 *
 *                     "new-instance" allocates a new instance of class
 *                     Rot2d and defines default axis, curve, rotation
 *                     angle, number of segments and point to edit.
 *
 *                     "delete" removes the Rot2d instance (if the ref-
 *                     count is zero) after freeing axis and curve.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1995  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


ROT2D *rot2d_new_instance(char *name)
{
  CLASS *class;
  ROT2D *self;
  TRIANG1D *curve, *axis;

  /*
   * START_METHOD returns the object the method was called on.
   * "new-instance" is always a class method, therefore G_CLASS
   * is used as parameter.
   */
  class = (CLASS *)START_METHOD(G_CLASS);

  /*
   * The return value is checked because the method could have
   * been called on an instance, then START_METHOD returns NULL.
   * If an error occurs a method always should return NULL, so
   * this is what we do if we get no class.
   */
  ASSURE(class, "", END_METHOD(NULL));

  /*
   * First we have to allocate memory for our instance, we call
   * "new-instance" on our superclass (using ^). Again we check
   * the return value to see if enough memory was available.
   */
  self = (ROT2D *)GRAPE(class, "^new-instance")(name);
  ASSURE(self, "rot2d_new_instance: can't create instance", END_METHOD(NULL));

  /*
   * We allocate memory for curve and axis by calling "new-instance"
   * on their class Triang1d, again these checks...
   */
  curve = self->curve = (TRIANG1D *)GRAPE(Triang1d, "new-instance")("curve");
  axis = self->axis = (TRIANG1D *)GRAPE(Triang1d, "new-instance")("axis");
  ASSURE(curve && axis,
	 "rot2d_new_instance: can't create curve/axis",
	 { GRAPE(self, "delete")(); END_METHOD(NULL); });

  /*
   * Only the structure for curve/axis was allocated by "new-instance".
   * We now use "list-alloc" to allocate memory for the coordinate lists,
   * in this case for three/two points. We define our points and set the
   * number of points to the right value.
   */
  GRAPE(curve, "list-alloc")(3);
  curve->x[0] =  0.5; curve->x[1] =  0.5; curve->x[2] =  0.5;
  curve->y[0] = -0.5; curve->y[1] =  0.0; curve->y[2] =  0.5;
  curve->z[0] =  0.0; curve->z[1] =  0.0; curve->z[2] =  0.0;
  curve->number_of_points = 3;

  GRAPE(axis, "list-alloc")(2);
  axis->x[0] =  0.0; axis->x[1] =  0.0;
  axis->y[0] = -1.0; axis->y[1] =  1.0;
  axis->z[0] =  0.0; axis->z[1] =  0.0;  
  axis->number_of_points = 2;

  /*
   * The other instance variables get default values, too. The
   * default point for editing is the middle point of the curve.
   */
  self->angle = 360.0;
  self->discr = 24;

  self->current = 1;

  /*
   * We return the instance we have created.
   */
  END_METHOD(self);
}


ROT2D *rot2d_free()
{
  ROT2D *self;

  /*
   * This is an instance method, therefore G_INSTANCE is used.
   * Again the return value is checked, somebody might have called
   * "free" on the class Rot2d (or a subclass).
   */
  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * First we have to delete curve and axis...
   */
  if(self->curve)
    GRAPE(self->curve, "delete")();
  if(self->axis)
    GRAPE(self->axis, "delete")();

  /*
   * Now calling "free" on the superclass will free any memory
   * allocated for the surface of revolution (this is done by
   * Triang2d "free") and the memory for the instance struct
   * (Triang2d "free" calls "free" on its superclass, too).
   */
  GRAPE(self, "^free")();

  /*
   * Free methods should always return NULL...
   */
  END_METHOD(NULL);
}
