/*
 * declaration-module for project "rotate"
 *
 * Description       : Setup-, add- and init-method for project rotate.
 *
 *                     "setup" declares what is used in the project,
 *                     "init" puts the user interface into the project and
 *                     "add" puts a Rot2d instance in the data hierarchy.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1995  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


/*
 * The class pointer for our new class
 */
CLASS *Rot2d = NULL;


/*
 * The add- and the init-function for the project, defined below.
 */
PROJECT *project_rotate_add();
PROJECT *project_rotate_init();

/*
 * Just to make life easier. We have to cast the functions pointers
 * because there is no type `pointer to an arbitrary function' in C.
 */
typedef void *(*FUNC)();


/*
 * List of all functions for our project. This list is passed to the
 * project in the "setup" method, the functions will be added to
 * the system when the project is used.
 */
static METHOD_DESCR rotate_methods[] = {
  { &Rot2d, "new-instance",       (FUNC)rot2d_new_instance      },
  { &Rot2d, "free",               (FUNC)rot2d_free              },
  { &Rot2d, "hardcopy",           (FUNC)rot2d_hardcopy          },
  { &Rot2d, "softcopy",           (FUNC)rot2d_softcopy          },
  { &Rot2d, "rotate-send",        (FUNC)rot2d_rotate_send       },
  { &Rot2d, "display",            (FUNC)rot2d_display           },
  { &Rot2d, "get-info",           (FUNC)rot2d_get_info          },
  { &Rot2d, "follow",             (FUNC)rot2d_follow            },
  { &Rot2d, "xdr",                (FUNC)rot2d_xdr               },
  { &Rot2d, "get-editor",         (FUNC)rot2d_get_editor        },
  { &Rot2d, "update-point",       (FUNC)rot2d_update_point      },
  { &Rot2d, "update-editor",      (FUNC)rot2d_update_editor     },
  { &Rot2d, "insert-point-send",  (FUNC)rot2d_insert_point_send },
  { &Rot2d, "delete-point-send",  (FUNC)rot2d_delete_point_send },

  { &Triang1d, "insert-point", (FUNC)triang1d_insert_point },
  { &Triang1d, "delete-point", (FUNC)triang1d_delete_point },

  { &Project, "rotate-add", (FUNC)project_rotate_add },
  { &Project, "rotate-init", (FUNC)project_rotate_init },

  { NULL, NULL, NULL }
};


PROJECT *project_rotate_setup()
{
  PROJECT *self;

  self = (PROJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * "setup" called on the project defines what classes, methods
   * and items (rulers, buttons etc) are used by the project.
   * We only add one class, therefore we don't use a class list,
   * but specify it directly. The version and date will be used
   * for the message in the manager menu.
   *
   * Note: The user-interface can not be created in this method,
   * because neither the class Rot2d is initialized nor the method
   * "get-editor" added to it. The editor is therefore added to
   * the project in the init method.
   */
  GRAPE(self, "setup")(P_Class, &Triang2d, &Rot2d, "Rot2d", sizeof(ROT2D),
		       P_MethodList, rotate_methods,
		       P_AddMethod, "rotate-add",
		       P_InitMethod, "rotate-init",
		       P_Version, 2, 0,
		       P_Date, "Aug 18, 96",
		       P_End);

  END_METHOD(self);
}


PROJECT *project_rotate_add()
{
  PROJECT *self;
  ROT2D *rot;

  self = (PROJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * This method is called when a project is added from the project list
   * in the manager menu or when g_project_add is called. We just create
   * a Rot2d instance like it was done in the main program.
   */
  rot = (ROT2D *)GRAPE(Rot2d, "new-instance")("rot");
  ASSURE(rot, "project_rotate_add: can't create instance", END_METHOD(NULL));

  /*
   * We set the instance as the project's object. The project will
   * put it into the data hierarchy for us, creating a scene if
   * necessary. By added the project several time we can get any
   * number of Rot2d instances.
   */
   GRAPE(self, "set-object")(rot);

  END_METHOD(self);
}


PROJECT *project_rotate_init(int batch)
{
  PROJECT *self;
  static GROUP *editor = NULL;

  self = (PROJECT *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * The init method is called when the project is used for the first
   * time. Here all classes are initialized and all methods added,
   * now we can create the editor (just once, therefore it has to be
   * static). The project might be called in batch mode, in this
   * case no interface is needed and we don't create it.
   */
  if(!batch && !editor) {
    /*
     * Again we need the manager, but not to add the editor to it...
     */
    MANAGER *mgr = (MANAGER *)GRAPE(Manager, "get-stdmgr")();

    /*
     * We create the editor, might fail...
     */
    ASSURE(editor = (GROUP *)GRAPE(Rot2d, "get-editor")(),
	   "project_rotate_init: can't get editor", END_METHOD(NULL));

    /*
     * This time we add the editor to the project instead of the
     * manager. The project then takes care of adding the editor
     * to or removing it from the manager when the project is added
     * or removed.
     */
    GRAPE(self, "add-item")(editor);

    /*
     * We tell the manager that it should send the method "update-
     * editor" to the class Rot2d everytime its current object is
     * changed. If the new current object has class Rot2d then the
     * editor is bound to that instance, otherwise the editor is
     * deactivated.
     */
    GRAPE(mgr, "add-cross-depend")(Rot2d, "update-editor");
  }

  END_METHOD(self);
}
