/*
 * grape-module "rot2d_point"
 *
 * Description       : Methods for adding a point to and removing a
 *                     point from a Triang1d instance and the interface
 *                     method to do this for a Rot2d instance curve.
 *
 * Author            : Bernd Oberknapp
 *
 * Copyright (c) 1995  by Universitaet Bonn
 *                        Institut fuer Angewandte Mathematik
 *                        Sonderforschungsbereich 256
 *                        D-53115 Bonn
 *
 * $Log:$
 */

#include <grape.h>
#include "rot2d.h"


TRIANG1D *triang1d_insert_point(int number)
{
  TRIANG1D *self;
  int i;

  self = (TRIANG1D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * First some checks. Since we insert the new point at position
   * "number", this index can not be the first point, it has to be
   * in [1, number of points - 1].
   */
  if(number <= 0 || number > self->number_of_points - 1) {
    fprintf(stderr,
	    "triang1d_insert_point: can't insert point at position %d\n",
	    number);
    END_METHOD(NULL);
  }

  /*
   * If we have already used all points in this triang1d instance we
   * need new memory for the new point. "list-alloc" will do the job
   * because for Triang1d it will copy the coordinates to the new
   * arrays, therefore we lose no information and get more memory!
   * Be careful: there are still some "list-alloc" methods (e.g. on
   * List_Of_Inst) which will not preserve the old data.
   */
  if(self->number_of_points == self->max_number_of_points)
    GRAPE(self, "list-alloc")(self->max_number_of_points + 1);

  /*
   * We have to shift the points from index number on to
   * get a free place for storing the new point (memmove
   * would probably be faster).
   */
  for(i = self->number_of_points - 1; i >= number; i--) {
    self->x[i + 1] = self->x[i];
    self->y[i + 1] = self->y[i];
    self->z[i + 1] = self->z[i];
  }

  /*
   * The new point is interpolated from the two points next to it,
   * its just the midpoint of the line between these points
   */
  self->x[number] = (self->x[number - 1] + self->x[number + 1]) / 2.0;
  self->y[number] = (self->y[number - 1] + self->y[number + 1]) / 2.0;
  self->z[number] = (self->z[number - 1] + self->z[number + 1]) / 2.0;

  /*
   * Now we have an additional point...
   */
  self->number_of_points++;

  END_METHOD(self);
}


TRIANG1D *triang1d_delete_point(int number)
{
  TRIANG1D *self;
  int i;

  self = (TRIANG1D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * Again first the checks. We don't want curves with no points or just
   * one point, and we have to make sure that the index number is valid.
   */
  if(self->number_of_points <= 2) {
    fprintf(stderr, "triang1d_delete_point: less than three points left\n");
    END_METHOD(NULL);
  }
  if(number < 0 || number > self->number_of_points - 1) {
    fprintf(stderr, "triang1d_delete_point: "
	            "can't delete point at position %d\n", number);
    END_METHOD(NULL);
  }

  /*
   * Like in "insert-point" we shift the points from index number
   * on, but now in the other directions. And we have one points less.
   */
  for(i = number; i < self->number_of_points - 1; i++) {
    self->x[i] = self->x[i + 1];
    self->y[i] = self->y[i + 1];
    self->z[i] = self->z[i + 1];
  }
  self->number_of_points--;

  END_METHOD(self);
}


ROT2D *rot2d_insert_point_send()
{
  ROT2D *self;

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * We definitely need a curve to insert a point.
   */
  if(!self->curve || !self->curve->number_of_points)
    END_METHOD(NULL);

  /*
   * We just call "insert-point" on the curve with the current
   * point as parameter to insert the point. Now the coordinate
   * rulers should show the coordinates of the new point,
   * therefore we update them.
   */
  if(GRAPE(self->curve, "insert-point")(self->current))
    GRAPE(Rot2d, "update-editor")(self);

  END_METHOD(self);
}


ROT2D *rot2d_delete_point_send()
{
  ROT2D *self;

  self = (ROT2D *)START_METHOD(G_INSTANCE);
  ASSURE(self, "", END_METHOD(NULL));

  /*
   * We need a curve for delete, too.
   */
  if(!self->curve || !self->curve->number_of_points)
    END_METHOD(NULL);

  /*
   * Just like insert, again we have to update the rulers,
   * because we have changed the current point.
   */
  if(GRAPE(self->curve, "delete-point")(self->current))
    GRAPE(Rot2d, "update-editor")(self);

  END_METHOD(self);
}
